@@ -1,13 +1,17 @@
+import datetime
+import re
+from inspect import getframeinfo, stack
+from uuid import UUID
+
+from orjson import JSONDecodeError
+
 from requests import codes as status_codes
 
+from .constants import (EMAIL_REGEX, HASH_REGEX, PHONE_REGEX, TOKEN_REGEX,
+                        URI_REGEX, URL_REGEX)
 from .exceptions import ValidationException
-import json
-import datetime
 from .utils.datetime import Now
-from uuid import UUID
-import re
-from .constants import HASH_REGEX, URI_REGEX, EMAIL_REGEX, PHONE_REGEX, TOKEN_REGEX, URL_REGEX
-from inspect import getframeinfo, stack
+from .utils.encoders import envoxy_json_loads
 
 DEFAULT_STATUS_CODE = status_codes.precondition_failed
 INVALID_TYPE_ERROR_CODE = 1202
@@ -21,7 +25,9 @@ def assertz(_expression, _error_message, _error_code, _status_code):
     :param _status_code: HTTP status code to be replied to the invoking HTTP client
     """
     if not _expression:
-        raise ValidationException(_error_message, code=_error_code, status=_status_code)
+        raise ValidationException(
+            _error_message, code=_error_code, status=_status_code)
+
 
 def assertz_reply(_expression, _error_msg, _error_code, _status_code):
     """
@@ -46,6 +52,7 @@ def assertz_reply(_expression, _error_msg, _error_code, _status_code):
             }
         }
 
+
 def assertz_call(_expression, _error_msg, _error_code, _status_code, reply=False):
     if not reply:
         return assertz(_expression, _error_msg, _error_code, _status_code)
@@ -62,13 +69,14 @@ def assertz_mandatory(_obj, _element=None, _error_code=1200, _status_code=DEFAUL
         return assertz_call(_element, "Key must not be emtpy", 1201, _status_code, reply=reply)
     elif _obj and _element is not None:
         return assertz_call(_element in _obj and _element is not None and _obj[_element] is not None, f"Mandatory: {_element}", _error_code,
-                _status_code, reply=reply)
+                            _status_code, reply=reply)
     else:
         return assertz_call(_obj, f"Mandatory: {_obj}", _error_code, _status_code, reply=reply)
 
 
 def assertz_string(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
-    if _element is None: return None
+    if _element is None:
+        return None
 
     try:
         value = _element if key is None else _element[key]
@@ -80,48 +88,54 @@ def assertz_string(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _sta
     except (AttributeError, TypeError):
         return assertz_call(False, f"Invalid value type: {value}", _error_code, _status_code, reply=reply)
 
+
 def assertz_integer(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
-    if _element is None: return None
+    if _element is None:
+        return None
 
     if isinstance(_element, int) and key is None:
         return assertz_call(_element, f"Invalid value type: {_element}", _error_code, _status_code, reply=reply)
     else:
         return assertz_call(key in _element and isinstance(_element[key], int), f"Invalid value type: {_element[key]}", _error_code,
-                _status_code, reply=reply)
+                            _status_code, reply=reply)
 
 
 def assertz_float(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
-    if _element is None: return None
+    if _element is None:
+        return None
 
     if isinstance(_element, float) and key is None:
         return assertz_call(_element, f"Invalid value type: {_element}", _error_code, _status_code, reply=reply)
     else:
         return assertz_call(key in _element and isinstance(_element[key], float), f"Invalid value type: {_element[key]}",
-                _error_code, _status_code, reply=reply)
+                            _error_code, _status_code, reply=reply)
 
 
 def assertz_timestamp(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
-    if _element is None or isinstance(_element, datetime.date) or isinstance(_element, datetime.date): return None
+    if _element is None or isinstance(_element, datetime.date) or isinstance(_element, datetime.date):
+        return None
 
     if key is None:
         return assertz_call(Now.to_datetime(_element), f"Invalid value type: {_element}", _error_code, _status_code, reply=reply)
     else:
         return assertz_call(key in _element and Now.to_datetime(_element[key]), f"Invalid value type: {_element[key]}", _error_code,
-                _status_code, reply=reply)
+                            _status_code, reply=reply)
 
 
 def assertz_boolean(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
-    if _element is None: return None
+    if _element is None:
+        return None
 
     if isinstance(_element, bool) and key is None:
         return assertz_call(_element in [True, False], f"Invalid value type: {_element}", _error_code, _status_code, reply=reply)
     else:
         return assertz_call(key in _element and isinstance(_element[key], bool), f"Invalid value type: {_element[key]}",
-                _error_code, _status_code, reply=reply)
+                            _error_code, _status_code, reply=reply)
 
 
 def assertz_array(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
-    if _element is None: return None
+    if _element is None:
+        return None
 
     try:
         value = _element if key is None else _element[key]
@@ -134,8 +148,24 @@ def assertz_array(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _stat
         return assertz_call(False, f"Invalid value type: {value}", _error_code, _status_code, reply=reply)
 
 
+def assertz_array_even_empty(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
+    if _element is None:
+        return None
+
+    try:
+        value = _element if key is None else _element[key]
+    except KeyError:
+        return assertz_call(False, f"Invalid value type: {key}", _error_code, _status_code, reply=reply)
+
+    try:
+        return assertz_call(isinstance(value, list), f"Invalid value type: {value}", _error_code, _status_code, reply=reply)
+    except TypeError:
+        return assertz_call(False, f"Invalid value type: {value}", _error_code, _status_code, reply=reply)
+
+
 def assertz_dict(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
-    if _element is None: return None
+    if _element is None:
+        return None
 
     try:
         value = _element if key is None else _element[key]
@@ -151,7 +181,8 @@ def assertz_dict(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _statu
 
 
 def assertz_json(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
-    if _element is None: return None
+    if _element is None:
+        return None
 
     try:
         value = _element if key is None else _element[key]
@@ -159,15 +190,14 @@ def assertz_json(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _statu
         return assertz_call(False, f"Invalid value type: {key}", _error_code, _status_code, reply=reply)
 
     try:
-        return assertz_call(json.loads(value), f"Invalid value type: {value}", _error_code, _status_code, reply=reply)
-
-    except (json.JSONDecodeError, TypeError):
-
+        return assertz_call(envoxy_json_loads(value), f"Invalid value type: {value}", _error_code, _status_code, reply=reply)
+    except (TypeError, JSONDecodeError):
         return assertz_call(False, f"Invalid value type: {value}", _error_code, _status_code, reply=reply)
 
 
 def assertz_complex(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
-    if _element is None: return None
+    if _element is None:
+        return None
 
     msg = ""
 
@@ -193,7 +223,8 @@ def assertz_complex(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _st
 
 
 def assertz_uuid(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
-    if _element is None: return None
+    if _element is None:
+        return None
 
     try:
         value = _element if key is None else _element[key]
@@ -209,7 +240,8 @@ def assertz_uuid(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _statu
 
 
 def assertz_utf8(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
-    if _element is None: return None
+    if _element is None:
+        return None
 
     _error_msg = "Invalid utf-8 encoding"
 
@@ -226,7 +258,8 @@ def assertz_utf8(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _statu
 
 
 def assertz_ascii(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
-    if _element is None: return None
+    if _element is None:
+        return None
 
     _error_msg = "Invalid ascii encoding"
 
@@ -240,8 +273,10 @@ def assertz_ascii(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _stat
     except (UnicodeEncodeError, AttributeError):
         return assertz_call(False, _error_msg, _error_code, _status_code, reply=reply)
 
+
 def assertz_regex(regex_expr, _error_msg, _element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
-    if _element is None: return None
+    if _element is None:
+        return None
 
     try:
         value = _element if key is None else _element[key]
@@ -255,23 +290,30 @@ def assertz_regex(regex_expr, _error_msg, _element, key=None, _error_code=INVALI
     except (AttributeError, TypeError):
         return assertz_call(False, _error_msg, _error_code, _status_code, reply=reply)
 
+
 def assertz_hash(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
     return assertz_regex(HASH_REGEX, "Invalid hash", _element, key, _error_code, _status_code, reply=reply)
 
+
 def assertz_token(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
     return assertz_regex(TOKEN_REGEX, "Invalid token", _element, key, _error_code, _status_code, reply=reply)
 
+
 def assertz_uri(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
     return assertz_regex(URI_REGEX, "Invalid uri", _element, key, _error_code, _status_code, reply=reply)
 
+
 def assertz_url(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
     return assertz_regex(URL_REGEX, "Invalid url", _element, key, _error_code, _status_code, reply=reply)
 
+
 def assertz_email(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
     return assertz_regex(EMAIL_REGEX, "Invalid email", _element, key, _error_code, _status_code, reply=reply)
 
+
 def assertz_location(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
-    if _element is None: return None
+    if _element is None:
+        return None
 
     _error_msg = "Invalid location"
 
@@ -290,15 +332,14 @@ def assertz_location(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _s
 
         elif isinstance(value, list):
             _expression = len(value) == 2 \
-                          and (isinstance(value[0], int) or isinstance(value[0], float)) \
-                          and (isinstance(value[1], int) or isinstance(value[1], float))
+                and (isinstance(value[0], int) or isinstance(value[0], float)) \
+                and (isinstance(value[1], int) or isinstance(value[1], float))
 
         else:
             raise TypeError
 
         return assertz_call(_expression, _error_msg, _error_code, _status_code, reply=reply)
 
-
     except (AttributeError, TypeError):
         return assertz_call(False, _error_msg, _error_code, _status_code, reply=reply)
 
@@ -306,6 +347,7 @@ def assertz_location(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _s
 def assertz_phone(_element, key=None, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
     return assertz_regex(PHONE_REGEX, "Invalid phone", _element, key, _error_code, _status_code, reply=reply)
 
+
 def assertz_intersects(x, y, _error_code=INVALID_TYPE_ERROR_CODE, _status_code=DEFAULT_STATUS_CODE, reply=False):
     _expression = set(x).intersection(y)
 
@@ -323,67 +365,86 @@ def assertz_valid_values(_expression, _error_msg, _error_code=INVALID_TYPE_ERROR
 def assertz_mandatory_reply(_element, key, _error_code, _status_code):
     return assertz_mandatory(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_string_reply(_element, key, _error_code, _status_code):
     return assertz_string(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_integer_reply(_element, key, _error_code, _status_code):
     return assertz_integer(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_float_reply(_element, key, _error_code, _status_code):
     return assertz_float(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_timestamp_reply(_element, key, _error_code, _status_code):
     return assertz_timestamp(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_boolean_reply(_element, key, _error_code, _status_code):
     return assertz_boolean(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_complex_reply(_element, key, _error_code, _status_code):
     return assertz_complex(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_dict_reply(_element, key, _error_code, _status_code):
     return assertz_dict(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_json_reply(_element, key, _error_code, _status_code):
     return assertz_json(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_array_reply(_element, key, _error_code, _status_code):
     return assertz_array(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_uuid_reply(_element, key, _error_code, _status_code):
     return assertz_uuid(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_utf8_reply(_element, key, _error_code, _status_code):
     return assertz_utf8(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_ascii_reply(_element, key, _error_code, _status_code):
     return assertz_ascii(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_hash_reply(_element, key, _error_code, _status_code):
     return assertz_hash(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_token_reply(_element, key, _error_code, _status_code):
     return assertz_token(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_uri_reply(_element, key, _error_code, _status_code):
     return assertz_uri(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_email_reply(_element, key, _error_code, _status_code):
     return assertz_email(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_location_reply(_element, key, _error_code, _status_code):
     return assertz_location(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_phone_reply(_element, key, _error_code, _status_code):
     return assertz_phone(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_intersects_reply(_element, key, _error_code, _status_code):
     return assertz_intersects(_element, key, _error_code, _status_code, reply=True)
 
+
 def assertz_unauthorized_reply(_element, key, _error_code):
     return assertz_unauthorized(_element, key, _error_code, reply=True)
 
+
 def assertz_valid_values_reply(_element, key, _error_code, _status_code):
     return assertz_valid_values(_element, key, _error_code, _status_code, reply=True)
-
-
